<?php
namespace Soisy\PaymentMethod\Helper;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Cms\Helper\Page as CmsHelper;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

class Settings extends AbstractHelper
{
    const SOISY_PAYMENT_CODE = 'soisy';
    const SOISY_SANDBOX_WEB_APP = 'https://shop.sandbox.soisy.it';
    const SOISY_SANDBOX_ENDPOINT = 'https://api.sandbox.soisy.it';
    const SOISY_SANDBOX_SHOP_ID_WIDGET_SIMULATION = 'soisytests';
    const SOISY_SANDBOX_SHOP_ID = 'partnershop';
    const SOISY_SANDBOX_X_AUTH_TOKEN = 'partnerkey';
    const SOISY_PRODUCTION_WEB_APP = 'https://shop.soisy.it';
    const SOISY_PRODUCTION_ENDPOINT = 'https://api.soisy.it';

    const XML_PATH_SETTINGS = 'payment/soisy/';

    const SOISY_ZERO_RATE_DISABLE_CONF = 'disable';
    const SOISY_ZERO_RATE_QUOTE_CONF = 'quote';
    const SOISY_ZERO_RATE_ITEM_CONF = 'item';
    const SOISY_ZERO_RATE_ITEM_MIX_CODE = 'mix';
    const SOISY_ZERO_RATE_ITEM_FIX_CODE = 'fix';

    protected $cmsHelper;
    protected $storeManager;
    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    public function __construct(
        Context               $context,
        CmsHelper             $cmsHelper,
        StoreManagerInterface $storeManager,
        ProductRepositoryInterface $productRepository
    )
    {
        parent::__construct($context);
        $this->cmsHelper = $cmsHelper;
        $this->storeManager = $storeManager;
        $this->productRepository = $productRepository;
    }

    public function getConfigValue($field, $storeId = null)
    {
        return $this->scopeConfig->getValue(
            $field,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    public function getSettings($code, $storeId = null)
    {
        return $this->getConfigValue(self::XML_PATH_SETTINGS . $code, $storeId);
    }

    public function isActive()
    {
        return $this->getSettings('active');
    }

    public function isSandbox()
    {
        return $this->getSettings('sandbox');
    }

    public function getMinAmount()
    {
        return $this->getSettings('min_order_total');
    }

    public function getEndpoint()
    {
        if ($this->isSandbox()) {
            return self::SOISY_SANDBOX_ENDPOINT;
        }
        return self::SOISY_PRODUCTION_ENDPOINT;
    }

    public function getWebapp()
    {
        if ($this->isSandbox()) {
            return self::SOISY_SANDBOX_WEB_APP;
        }
        return self::SOISY_PRODUCTION_WEB_APP;
    }

    public function getShopId($isForWidget = false)
    {
        if ($this->isSandbox()) {
            if ($isForWidget) {
                return self::SOISY_SANDBOX_SHOP_ID_WIDGET_SIMULATION;
            } else {
                return self::SOISY_SANDBOX_SHOP_ID;
            }
        }
        return $this->getSettings('shop_id');
    }

    public function getAuthToken()
    {
        if ($this->isSandbox()) {
            return self::SOISY_SANDBOX_X_AUTH_TOKEN;
        }
        return $this->getSettings('auth_token');
    }

    public function getInstructions()
    {
        return $this->getSettings('instructions');
    }

    public function showSimulation()
    {
        return $this->getSettings('show_simulation');
    }

    public function getPromotionalRates()
    {
        return $this->getSettings('promotional_rates');
    }

    protected function getCmsUrl($urlString)
    {
        return $urlString ? $this->cmsHelper->getPageUrl($urlString) : $this->storeManager->getStore()->getBaseUrl();
    }

    public function getCmsSuccessUrl()
    {
        $url = $this->getSettings('cms_success_url');
        return $this->getCmsUrl($url);
    }

    public function getCmsErrorUrl()
    {
        $url = $this->getSettings('cms_error_url');
        return $this->getCmsUrl($url);
    }

    public function getZeroRateConfiguration()
    {
        return $this->getSettings('zero_rate');
    }

    public function getZeroRateQuoteThreshold()
    {
        return $this->getSettings('zero_rate_quote_threshold');
    }

    public function getZeroRateItemConfiguration()
    {
        return $this->getSettings('zero_rate_item');
    }

    public function getZeroRateItemAttributeCode()
    {
        return $this->getSettings('zero_rate_item_attribute');
    }

    /**
     * @param array $orderItems
     * @param float|int $total
     * @param int $storeId
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function checkZeroRate($orderItems, $total, $storeId)
    {
        $zeroRateConfig = $this->getZeroRateConfiguration();
        $applicableZeroRate = false;

        switch ($zeroRateConfig) {
            case self::SOISY_ZERO_RATE_QUOTE_CONF:
                if($total >= floatval($this->getZeroRateQuoteThreshold())) {
                    $applicableZeroRate = true;
                }
                break;
            case self::SOISY_ZERO_RATE_ITEM_CONF:
                $checkQuoteItem = $this->checkQuoteItemForZeroRate($orderItems, $this->getZeroRateItemConfiguration(), $storeId);
                if($checkQuoteItem) {
                    $applicableZeroRate = true;
                }
                break;
            case self::SOISY_ZERO_RATE_DISABLE_CONF:
                $applicableZeroRate = false;
                break;
            default:
                $applicableZeroRate = false;
        }

        return $applicableZeroRate;
    }

    /**
     * @param array $orderItem
     * @param string $itemConfigCode
     * @param int $storeId
     * @param bool $checkAllItem
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function checkQuoteItemForZeroRate($orderItem, $itemConfigCode, $storeId, $checkAllItem = false)
    {
        $result = false;
        $zeroRateAttributeCode = $this->getZeroRateItemAttributeCode();
        if(is_null($zeroRateAttributeCode) || $zeroRateAttributeCode == '') {
            return $result;
        }

        if($itemConfigCode == self::SOISY_ZERO_RATE_ITEM_MIX_CODE)
        {
            foreach ($orderItem as $item)
            {
                if($item->getPrice() > 0)
                {
                    $product = $this->loadProduct($item->getProduct(), $zeroRateAttributeCode, $storeId);
                    if($product->getData($zeroRateAttributeCode) == 1) {
                        $result = true;
                        break;
                    }
                }
            }
        }
        elseif ($itemConfigCode == self::SOISY_ZERO_RATE_ITEM_FIX_CODE)
        {
            $result = true;
            $zeroRateCounter = 0;
            foreach ($orderItem as $item)
            {
                if($item->getPrice() > 0)
                {
                    $product = $this->loadProduct($item->getProduct(), $zeroRateAttributeCode, $storeId);
                    if($product->getData($zeroRateAttributeCode) == 0) {
                        $result = false;
                        if(!$checkAllItem) {
                            return $result;
                        }
                    }
                    else {
                        $zeroRateCounter++;
                    }
                }
            }

            /*
             * If $checkAllItem is active and no items in cart have the zero rate attribute,
             * return true for don't stop the flow.
             */
            if (!$result && $zeroRateCounter == 0) {
                $result = true;
            }
        }

        return $result;
    }

    protected function loadProduct($product, $zeroRateAttributeCode, $storeId)
    {
        if(is_null($product->getData($zeroRateAttributeCode))) {
            $product = $this->productRepository->getById(
                $product->getId(),
                false,
                $storeId,
                true
            );
        }

        return $product;
    }
}
